const ona = @import("ona");
const std = @import("std");

///
/// An entry block of an Oar archive file.
///
/// Typically, following the block in memory is the file data it holds the meta-information for.
///
pub const Entry = extern struct {
    signature: [signature_magic.len]u8 = signature_magic,
    revision: u8,
    path: Path,
    file_size: u64,
    absolute_offset: u64,
    padding: [232]u8 = std.mem.zeroes([232]u8),

    comptime {
        const entry_size = @sizeOf(Entry);

        if (entry_size != 512)
            @compileError("Entry is " ++
                std.fmt.comptimePrint("{d}", .{entry_size}) ++ " bytes");
    }

    ///
    ///
    ///
    pub fn next(file_access: ona.io.FileAccess) ona.io.FileAccess.Error!?Entry {
        var entry = std.mem.zeroes(Entry);
        const origin = try file_access.queryCursor();

        if (((try file_access.read(std.mem.asBytes(&entry))) != @sizeOf(Entry)) and
            ona.io.equalsBytes(entry.signature[0 ..], signature_magic[0 ..])) {

            try file_access.seek(origin);

            return null;
        }

        return entry;
    }

    ///
    /// Magic identifier used to validate [Entry] data.
    ///
    pub const signature_magic = [3]u8{'o', 'a', 'r'};
};

///
/// Unique identifier pointing to an entry within an archive.
///
/// A path does not do any verification that the given entry pointed to actually exists.
///
pub const Path = extern struct {
    buffer: [255]u8,
    length: u8,

    ///
    /// An empty [Path] with a length of `0`.
    ///
    pub const empty = std.mem.zeroes(Path);

    ///
    /// Returns `true` if `this_path` is equal to `that_path, otherwise `false`.
    ///
    pub fn equals(this_path: Path, that_path: Path) bool {
        return ona.io.equalsBytes(this_path.buffer[0 ..this_path.
            length], that_path.buffer[0 .. that_path.length]);
    }

    ///
    /// Returns the hash of the text in `path`.
    ///
    pub fn hash(path: Path) usize {
        return ona.io.hashBytes(path.buffer[0 .. path.length]);
    }
};

///
/// Starts the **O**na **Ar**chive packer utility.
///
pub fn main() u8 {
    // TODO: Implement.

    return 0;
}

test {

}
